/******************************************************************************
 * 
 * CS105: Cachelab
 *
 * Starter code: caching.h
 *
 * Header file for cachelab assignment
 * Includes macros, type definitions, global vars, and function declarations
 *
 * You DO NOT need to modify this file.
 *
 *****************************************************************************/
#ifndef CACHING_H_
#define CACHING_H_

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#define NUM_CACHELINES 256 // 256 cachelines in the simulated cache
#define DATA_BLOCK_LEN 256 // 256 bytes in each data block

typedef unsigned char byte;
typedef unsigned char bool;
typedef struct {
  bool valid; // 1 bit valid (actually 8 bits, but we'll pretend)
  long tag; // 48 bit tag (actually 64 bits, but we'll pretend)
  byte datablock[DATA_BLOCK_LEN]; // 256 byte data blocks
} cacheline_t;

// store pointer to cache as global variables
extern cacheline_t* direct_cache;


/******************************************************************************
 * parse_addr takes four arguments. The first argument (addr) is a (64-bit) 
 * address to parse. The other three arguments are addresses for where to 
 * write the parsed tag, index, and offset (respectively).
 *****************************************************************************/
void parse_addr(void* addr, long* tag_ptr, byte* index_ptr, byte* offset_ptr);


/******************************************************************************
 * is_in_direct_cache takes three arguments: tag, index, and offset and
 * returns 1 if the associated address is currently stored in the cache or 
 * 0 otherwise.
 *****************************************************************************/
bool is_in_direct_cache(long tag, byte index, byte offset);


/******************************************************************************
 * lookup_int_in_direct_cache takes three arguments: tag, index, and offset
 * and returns the integer value stored in cache at the corresponding address
 * 
 * Precondition: this function should only be called if the data at that
 * address is currently in the cache
 *****************************************************************************/
int lookup_int_in_direct_cache(long tag, byte index, byte offset);


/******************************************************************************
 * update_direct_cache modifies the cache by overwriting the cacheline
 * at index to respond to the cache miss for addr
 *
 * Precondition: this function should only be called if the address is NOT
 * currently in the cache
 *****************************************************************************/
void update_direct_cache(long tag, byte index, byte offset, void* addr);

#endif
